﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Device.Location;


namespace Location
{
    public partial class MainPage : PhoneApplicationPage
    {
        GeoCoordinateWatcher watcher;

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            stopLocationButton.IsEnabled = false;            
        }

        private void startLocationButton_Click(object sender, RoutedEventArgs e)
        {
            if (watcher == null)
            {
                watcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High);
                watcher.MovementThreshold = 20; // compensate for signal noise
                watcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>(watcher_StatusChanged);
                watcher.PositionChanged += new EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>(watcher_PositionChanged);
                watcher.Start();
            }
        }

        void watcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            latitudeText.Text = e.Position.Location.Latitude.ToString("0.000");
            longitudeText.Text = e.Position.Location.Longitude.ToString("0.000");
            altitudeText.Text = e.Position.Location.Altitude.ToString("00000.00");
        }

        void watcher_StatusChanged(object sender, GeoPositionStatusChangedEventArgs e)
        {
            switch (e.Status)
            {
                case GeoPositionStatus.Disabled:
                    if (watcher.Permission == GeoPositionPermission.Denied)
                    {
                        watcherStatus.Text = "Application is not allowed to use location services.";
                    }
                    else
                    {
                        watcherStatus.Text = "Location services not working properly.";
                    }
                    break;
                case GeoPositionStatus.Initializing:
                    startLocationButton.IsEnabled = false;
                    watcherStatus.Text = "Initializing...";
                    break;
                case GeoPositionStatus.NoData:
                    stopLocationButton.IsEnabled = true;
                    watcherStatus.Text = "Active, but no location data.";
                    break;
                case GeoPositionStatus.Ready:
                    watcherStatus.Text = "Location data is available.";
                    stopLocationButton.IsEnabled = true;
                    break;
            }
        }

        private void stopLocationButton_Click(object sender, RoutedEventArgs e)
        {
            watcher.Stop();
            watcherStatus.Text = "(not monitoring GPS)";
        }
    }
}