﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;
using System.Net;
using System.IO;
using System.Runtime.Serialization;
using SharedData;

namespace UsVsThemConsole
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string[] simUris;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void SendTile(string[] uriTargets, string backgroundImage, int count, string title)
        {
            Console.WriteLine("About to send tile to the following URIs:");

            foreach (string uri in uriTargets)
            {                
                Console.WriteLine(uri);
                HttpWebRequest sendNotificationRequest = (HttpWebRequest)WebRequest.Create(uri);

                // HTTP POST is the only allowed method to send the notification.
                sendNotificationRequest.Method = "POST";
                sendNotificationRequest.ContentType = "text/xml";
                sendNotificationRequest.Headers.Add("X-WindowsPhone-Target", "token");
                sendNotificationRequest.Headers.Add("X-NotificationClass", "1");

                string payloadString = string.Format("<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
                    "<wp:Notification xmlns:wp=\"WPNotification\">" +
                    "<wp:Tile>" +
                        "<wp:BackgroundImage>{0}</wp:BackgroundImage>" +
                        "<wp:Count>{1}</wp:Count>" +
                        "<wp:Title>{2}</wp:Title>" +
                    "</wp:Tile>" +
                    "</wp:Notification>", backgroundImage, count.ToString(), title);

                byte[] notificationMessage = System.Text.UTF8Encoding.UTF8.GetBytes(payloadString);
                // Sets the web request content length.
                sendNotificationRequest.ContentLength = notificationMessage.Length;

                // Sets the notification payload to send.                
                using (Stream requestStream = sendNotificationRequest.GetRequestStream())
                {
                    requestStream.Write(notificationMessage, 0, notificationMessage.Length);
                }

                // Sends the notification and gets the response.
                HttpWebResponse response = (HttpWebResponse)sendNotificationRequest.GetResponse();
                string notificationStatus = response.Headers["X-NotificationStatus"];
                string notificationChannelStatus = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
            }
        }

        

        private void SendToast(string[] uriTargets, string text1, string text2)
        {
            Console.WriteLine("About to send to the following URIs:");

            foreach (string uri in uriTargets)
            {
                string subscriptionUri = uri;
                Console.WriteLine(uri);
                HttpWebRequest sendNotificationRequest = (HttpWebRequest)WebRequest.Create(subscriptionUri);

                // HTTP POST is the only allowed method to send the notification.
                sendNotificationRequest.Method = "POST";
                sendNotificationRequest.ContentType = "text/xml";
                sendNotificationRequest.Headers.Add("X-WindowsPhone-Target", "toast");
                sendNotificationRequest.Headers.Add("X-NotificationClass", "2");

                string payloadString = string.Format("<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
                    "<wp:Notification xmlns:wp=\"WPNotification\">" +
                    "<wp:Toast>" +
                        "<wp:Text1>{0}</wp:Text1>" +
                        "<wp:Text2>{1}</wp:Text2>" +
                    "</wp:Toast>" +
                    "</wp:Notification>", text1, text2);

                byte[] notificationMessage = System.Text.UTF8Encoding.UTF8.GetBytes(payloadString);
                // Sets the web request content length.
                sendNotificationRequest.ContentLength = notificationMessage.Length;

                // Sets the notification payload to send.                
                using (Stream requestStream = sendNotificationRequest.GetRequestStream())
                {
                    requestStream.Write(notificationMessage, 0, notificationMessage.Length);
                }

                // Sends the notification and gets the response.
                HttpWebResponse response = (HttpWebResponse)sendNotificationRequest.GetResponse();
                string notificationStatus = response.Headers["X-NotificationStatus"];
                string notificationChannelStatus = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
            }
        }

        private void sendToastButton_Click(object sender, RoutedEventArgs e)
        {
            NotificationService.NotificationsClient client = new NotificationService.NotificationsClient();

            string[] uris = client.GetRegisteredChannels();

            SendToast(uris, toastLine1.Text, toastLine2.Text);
           
        }

        private void sendRawButton_Click(object sender, RoutedEventArgs e)
        {
            ZombiePopulationPayload zombiePopulation = new ZombiePopulationPayload() { Humans = 5.25m, Zombies = 8.23m };
            DataContractSerializer ser = new DataContractSerializer(typeof(ZombiePopulationPayload));
            MemoryStream ms = new MemoryStream();
            ser.WriteObject(ms, zombiePopulation);
            ms.Seek(0, SeekOrigin.Begin);
            byte[] zombieBytes = ms.GetBuffer();

            NotificationService.NotificationsClient client = new NotificationService.NotificationsClient();

            string[] uris = client.GetRegisteredChannels();


            Console.WriteLine("About to send raw data to the following URIs:");

            foreach (string uri in uris)
            {


                HttpWebRequest sendNotificationRequest = (HttpWebRequest)WebRequest.Create(uri);

                // HTTP POST is the only allowed method to send the notification.
                sendNotificationRequest.Method = "POST";
                sendNotificationRequest.Headers.Add("X-NotificationClass", "3");

                // Sets the web request content length.
                sendNotificationRequest.ContentLength = zombieBytes.Length;

                // Sets the notification payload to send.
                using (Stream requestStream = sendNotificationRequest.GetRequestStream())
                {
                    requestStream.Write(zombieBytes, 0, zombieBytes.Length);
                }

                // Sends the notification and gets the response.
                HttpWebResponse response = (HttpWebResponse)sendNotificationRequest.GetResponse();
                string notificationStatus = response.Headers["X-NotificationStatus"];
                string notificationChannelStatus = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
            }
        }

        private void simulateButton_Click(object sender, RoutedEventArgs e)
        {
            NotificationService.NotificationsClient client = new NotificationService.NotificationsClient();
            simUris = client.GetRegisteredChannels();

            System.Timers.Timer t = new System.Timers.Timer();
            t.Interval = 30000;
            t.Elapsed += new System.Timers.ElapsedEventHandler(t_Elapsed);
            t.Start();

            System.Timers.Timer toastTimer = new System.Timers.Timer();
            toastTimer.Interval = 20000;
            toastTimer.Elapsed += new System.Timers.ElapsedEventHandler(toastTimer_Elapsed);
            toastTimer.Start();
        }

        void toastTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            SendToast(simUris, "Us v Them", "Zombies on the move!");
        }

        void t_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {            
            Random r = new Random();
            int choice = r.Next(100);
            if (choice > 50) // zombies winning
            {
                SendTile(simUris, @"zombies.png", choice, "Zombies Winning!");
            }
            else // humans winning
            {
                SendTile(simUris, @"humans.png", choice, "Humans Winning!");
            }
        }
    }
}
