﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Notification;
using System.Diagnostics;
using System.Runtime.Serialization;
using SharedData;
using System.IO;
using System.Collections.ObjectModel;

namespace Us_vs_Them
{
    public partial class App : Application
    {
        /// <summary>
        /// Provides easy access to the root frame of the Phone Application.
        /// </summary>
        /// <returns>The root frame of the Phone Application.</returns>
        public PhoneApplicationFrame RootFrame { get; private set; }

        public HttpNotificationChannel NotificationChannel { get; private set; }
        private const string NOTIFY_CHANNEL_NAME = "us_vs_them_channel";
        private const string NOTIFY_SERVICE_NAME = "www.usversusthemsampleapp.com";
        /// <summary>
        /// Constructor for the Application object.
        /// </summary>
        public App()
        {
            // Global handler for uncaught exceptions. 
            UnhandledException += Application_UnhandledException;

            // Show graphics profiling information while debugging.
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // Display the current frame rate counters.
                Application.Current.Host.Settings.EnableFrameRateCounter = true;

                // Show the areas of the app that are being redrawn in each frame.
                //Application.Current.Host.Settings.EnableRedrawRegions = true;

                // Enable non-production analysis visualization mode, 
                // which shows areas of a page that are being GPU accelerated with a colored overlay.
                //Application.Current.Host.Settings.EnableCacheVisualization = true;
            }

            // Standard Silverlight initialization
            InitializeComponent();

            // Phone-specific initialization
            InitializePhoneApplication();

            // Notification initialization
            InitializeNotifications();
        }

        private void InitializeNotifications()
        {
            NotificationChannel = HttpNotificationChannel.Find(NOTIFY_CHANNEL_NAME);

            if (NotificationChannel == null)
            {
                NotificationChannel = new HttpNotificationChannel(NOTIFY_CHANNEL_NAME, NOTIFY_SERVICE_NAME);
                SetupDelegates();

                NotificationChannel.Open();
            }
            else
            {
                SetupDelegates();

                Debug.WriteLine("Found existing channel at {0}", NotificationChannel.ChannelUri.ToString());
            }


            SubscribeToNotifications();
            if (NotificationChannel.ChannelUri != null)
            {
                RegisterNotificationChannelWithService(NotificationChannel.ChannelUri.ToString());
            }
        }

        private void SubscribeToNotifications()
        {
            // Cannot receive toasts without binding our notification channel
            // to toasts. If we receive a toast and we have not bound our channel
            // to those toasts, a very hard to debug exception will occur.
            if (NotificationChannel.IsShellToastBound == true)
            {
                Debug.WriteLine("Already bound to Toast notification");
            }
            else
            {
                Debug.WriteLine("Registering to Toast Notifications");
                NotificationChannel.BindToShellToast();
            }

            if (NotificationChannel.IsShellTileBound == true)
            {
                Debug.WriteLine("Already bound to tile");
            }
            else
            {
                Debug.WriteLine("Registering to Tile Notification.");
                NotificationChannel.BindToShellTile();
            }
        }

        private void SetupDelegates()
        {
            NotificationChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(NotificationChannel_ChannelUriUpdated);            
            NotificationChannel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(NotificationChannel_HttpNotificationReceived);
            NotificationChannel.ShellToastNotificationReceived += new EventHandler<NotificationEventArgs>(NotificationChannel_ShellToastNotificationReceived);            
            NotificationChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(NotificationChannel_ErrorOccurred);
        }

        void NotificationChannel_ErrorOccurred(object sender, NotificationChannelErrorEventArgs e)
        {
            Debug.WriteLine(e.ErrorType.ToString() + ":" + e.ErrorCode.ToString() + ":\n" + e.Message);
            //throw new NotImplementedException();
        }

        void NotificationChannel_ShellToastNotificationReceived(object sender, NotificationEventArgs e)
        {
            string title = e.Collection.Values.First();
            string message = e.Collection.Values.Skip(1).FirstOrDefault() ?? string.Empty;
            Deployment.Current.Dispatcher.BeginInvoke(() => MessageBox.Show(message, title, MessageBoxButton.OK));
        }

        void NotificationChannel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
        {            
            if (e.Notification.Body != null && e.Notification.Headers != null)
            {
                // Thanks to Daniel Vaughn (WP7 Unleashed by SAMS Press)
                // for this tidbit on having to remove nulls from the serialized
                // data contract sent from the push app/service
                using (BinaryReader reader = new BinaryReader(e.Notification.Body))
                {
                    byte[] bodyBytes = reader.ReadBytes((int)e.Notification.Body.Length);

                    int lengthWithoutNulls = bodyBytes.Length;
                    for (int i = bodyBytes.Length - 1; i >= 0 && bodyBytes[i] == '\0';
                            i--, lengthWithoutNulls--)
                    {
                        /* Intentionally left blank. */
                    }

                    byte[] cleanedBytes = new byte[lengthWithoutNulls];
                    Array.Copy(bodyBytes, cleanedBytes, lengthWithoutNulls);

                    DataContractSerializer ser = new DataContractSerializer(typeof(ZombiePopulationPayload));
                    using (MemoryStream stream = new MemoryStream(cleanedBytes))
                    {
                        ZombiePopulationPayload zombiePopulation = (ZombiePopulationPayload)ser.ReadObject(stream);
                        Deployment.Current.Dispatcher.BeginInvoke( 
                            () =>
                                MessageBox.Show(string.Format("Us:{0} Them:{1}", 
                                zombiePopulation.Humans, zombiePopulation.Zombies), 
                                "Population Update", MessageBoxButton.OK)
                        );
                    }                                                                                
                }                                                
            }
        }

        void NotificationChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            Debug.WriteLine("Channel URI updated " + e.ChannelUri.ToString());
            RegisterNotificationChannelWithService(e.ChannelUri.ToString());
        }

        private void RegisterNotificationChannelWithService(string channelUri)
        {
            // proxy to my home-grown channel URI registration service
            NotificationService.NotificationsClient client = new NotificationService.NotificationsClient();
            client.RegisterChannelAsync(channelUri);
            client.RegisterChannelCompleted += new EventHandler<System.ComponentModel.AsyncCompletedEventArgs>(client_RegisterChannelCompleted);
        }

        void client_RegisterChannelCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                Debug.WriteLine("Registered channel URI with notification service.");
            }
        }

        // Code to execute when the application is launching (eg, from Start)
        // This code will not execute when the application is reactivated
        private void Application_Launching(object sender, LaunchingEventArgs e)
        {
        }

        // Code to execute when the application is activated (brought to foreground)
        // This code will not execute when the application is first launched
        private void Application_Activated(object sender, ActivatedEventArgs e)
        {
        }

        // Code to execute when the application is deactivated (sent to background)
        // This code will not execute when the application is closing
        private void Application_Deactivated(object sender, DeactivatedEventArgs e)
        {
        }

        // Code to execute when the application is closing (eg, user hit Back)
        // This code will not execute when the application is deactivated
        private void Application_Closing(object sender, ClosingEventArgs e)
        {
        }

        // Code to execute if a navigation fails
        private void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // A navigation has failed; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        // Code to execute on Unhandled Exceptions
        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        #region Phone application initialization

        // Avoid double-initialization
        private bool phoneApplicationInitialized = false;

        // Do not add any additional code to this method
        private void InitializePhoneApplication()
        {
            if (phoneApplicationInitialized)
                return;

            // Create the frame but don't set it as RootVisual yet; this allows the splash
            // screen to remain active until the application is ready to render.
            RootFrame = new PhoneApplicationFrame();
            RootFrame.Navigated += CompleteInitializePhoneApplication;

            // Handle navigation failures
            RootFrame.NavigationFailed += RootFrame_NavigationFailed;

            // Ensure we don't initialize again
            phoneApplicationInitialized = true;
        }

        // Do not add any additional code to this method
        private void CompleteInitializePhoneApplication(object sender, NavigationEventArgs e)
        {
            // Set the root visual to allow the application to render
            if (RootVisual != RootFrame)
                RootVisual = RootFrame;

            // Remove this handler since it is no longer needed
            RootFrame.Navigated -= CompleteInitializePhoneApplication;
        }

        #endregion
    }
}