﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using IsolatedStorageDemo.Models;


namespace IsolatedStorageDemo
{
    public class MainViewModel
    {
        public MainViewModel()
        {
            this.Items = new ObservableCollection<ItemViewModel>();
        }
        
        public ObservableCollection<ItemViewModel> Items { get; private set; }

        public bool IsDataLoaded
        {
            get;
            private set;
        }

        /// <summary>
        /// Loads the data to populate the view model
        /// </summary>
        public void LoadData()
        {
            LoadFromIso();
            this.IsDataLoaded = true;
        }

        public void LoadFromIso()
        {
            AppData appData = 
                IsoStoreHelper.ReadGraphFromFile<AppData>("appdata.dat", () =>
            {
                AppData seed = new AppData()
                {
                    Customers = new Customer[] {
                        new Customer { Name = "First Customer", 
                            Description = "This is the first customer" } }
                };
                return seed;
            });

            this.Items.Clear();
            foreach (Customer cust in appData.Customers)
            {
                this.Items.Add(new ItemViewModel { Name = cust.Name, 
                    Description = cust.Description });
            }
        }

        public void SaveToIso()
        {
            AppData data = new AppData();
            List<Customer> customerList = new List<Customer>();
            foreach (ItemViewModel itemView in this.Items)
            {
                customerList.Add(new Customer { Name = itemView.Name, 
                    Description = itemView.Description });
            }
            data.Customers = customerList.ToArray();
            IsoStoreHelper.WriteGraphToFile("appdata.dat", data);
        }        
    }
}