﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Net;
using System.IO;
using System.Xml;
using System.Linq;
using System.Xml.Linq;
using SmartClientDemo.ViewModels;


namespace SmartClientDemo
{
    public class MainViewModel : INotifyPropertyChanged
    {
        public MainViewModel()
        {
            this.Videos = new ObservableCollection<VideoViewModel>();
            this.BlogPosts = new ObservableCollection<BlogViewModel>();
            this.Tweets = new ObservableCollection<Tweet>();
        }

        /// <summary>
        /// A collection for ItemViewModel objects.
        /// </summary>
        public ObservableCollection<VideoViewModel> Videos { get; private set; }
        public ObservableCollection<BlogViewModel> BlogPosts { get; private set; }
        public ObservableCollection<Tweet> Tweets { get; private set; }

        public bool VideosLoaded
        {
            get;
            private set;
        }

        public bool BlogLoaded { get; private set; }
        public bool TweetsLoaded { get; private set; }


        /// <summary>
        /// Creates and adds a few ItemViewModel objects into the Items collection.
        /// </summary>
        public void LoadVideos()
        {
            // Sample data; replace with real data
            WebClient wc = new WebClient();
            wc.DownloadStringAsync(new Uri("http://gdata.youtube.com/feeds/api/users/thecrochetdude/uploads"));
            wc.DownloadStringCompleted += new DownloadStringCompletedEventHandler(wc_DownloadStringCompleted);        
            this.VideosLoaded = true;
        }

        public void LoadBlog()
        {
            WebClient wcBlog = new WebClient();
            wcBlog.DownloadStringAsync(new Uri("http://feeds.feedburner.com/ckindel?format=xml"));
            wcBlog.DownloadStringCompleted += new DownloadStringCompletedEventHandler(wcBlog_DownloadStringCompleted);
        }

        public void LoadTweets()
        {
            WebClient wcTweets = new WebClient();     
            wcTweets.DownloadStringAsync(new Uri("http://api.twitter.com/1/statuses/user_timeline.xml?screen_name=ckindel"));
            wcTweets.DownloadStringCompleted += new DownloadStringCompletedEventHandler(wcTweets_DownloadStringCompleted);
        }

        void wcTweets_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            StringReader sr = new StringReader(e.Result);
            var reader = XmlReader.Create(sr);
            var document = XDocument.Load(reader);

            var tweets =
                from tweet in document.Descendants("status")
                select tweet;

            App.ViewModel.Tweets.Clear();
            foreach (var tweet in tweets)
            {
                Tweet newTweet = new Tweet 
                { 
                    Text = (string)tweet.Element("text"),
                    CreatedAt = DateTime.ParseExact((string)tweet.Element("created_at"), "ddd MMM dd HH:mm:ss zz00 yyyy", null)
                };
                App.ViewModel.Tweets.Add(newTweet);
            }

            TweetsLoaded = true;
        }

        void wcBlog_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            StringReader sr = new StringReader(e.Result);
            var reader = XmlReader.Create(sr);
            var document = XDocument.Load(reader);

            var posts =
                from item in document.Descendants("item")
                select item;

            App.ViewModel.BlogPosts.Clear();
            foreach (var post in posts)
            {
                BlogViewModel blogpost = new BlogViewModel 
                {
                    Title = (string)post.Element("title"),
                    Description = (string)post.Element("description"), 
                    PublishDate = DateTime.Parse((string)post.Element("pubDate"))
                };
                App.ViewModel.BlogPosts.Add(blogpost);
            }

            BlogLoaded = true;
        }

        void wc_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            StringReader sr = new StringReader(e.Result);
            var reader = XmlReader.Create(sr);
            var document = XDocument.Load(reader);

            XNamespace atom = "http://www.w3.org/2005/Atom";
            var entries =
                from entry in document.Descendants(atom + "entry")                
                select entry;

            App.ViewModel.Videos.Clear();
            foreach (var entry in entries)
            {
                App.ViewModel.Videos.Add(new VideoViewModel { Title = (string)entry.Element(atom + "title"), Description = (string)entry.Element(atom+"content") });
                //Debug.WriteLine(entry.ToString());                
                //Debug.WriteLine("Author:" + (string)entry.Element(atom + "author").Element(atom + "name"));
            }

            VideosLoaded = true;
        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}