﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO.IsolatedStorage;
using System.IO;
using System.Runtime.Serialization;

namespace Mnemuse.Model
{
    public static class IsoStoreHelper
    {

        public static void WriteGraphToFile<T>(string filename, T sourceObject)
        {
            using (IsolatedStorageFile isf =
                IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (IsolatedStorageFileStream stream =
                  new IsolatedStorageFileStream(filename, FileMode.Create, isf))
                {
                    DataContractSerializer dcs = new DataContractSerializer(typeof(T));
                    dcs.WriteObject(stream, sourceObject);
                    stream.Close();
                }
            }
        }

        public static T ReadGraphFromFile<T>(string filename) where T:class
        {
            T result = null;
            using (IsolatedStorageFile isf = 
                IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (IsolatedStorageFileStream stream =
                    new IsolatedStorageFileStream(filename, FileMode.OpenOrCreate, isf))
                {
                    if (stream.Length > 0)
                    {
                        DataContractSerializer dcs = new DataContractSerializer(typeof(T));
                        result = dcs.ReadObject(stream) as T;
                    }
                }
            }

            return result;
        }

        public static T ReadGraphFromFile<T>(string filename, Func<T> seedFunc) where T : class
        {
            if (IsolatedStorageFile.GetUserStoreForApplication().FileExists(filename))
                return ReadGraphFromFile<T>(filename);
            else
                return seedFunc();
        }
    }
}
