﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Messaging;
using Mnemuse.Messages;

namespace Mnemuse.ViewModel
{
    public class Book : ViewModelBase
    {
        private string title;
        private string description;
        
        public Book()
            : base()
        {            
            InitializeCommands();            
        }

        public RelayCommand RemoveBookCommand { get; private set; }

        public string Title
        {
            get
            {
                return title;
            }
            set
            {
                title = value;
                RaisePropertyChanged("Title");
            }
        }

        // this is not a notify change property
        public Guid ID { get; set; }

        public string Description
        {
            get
            {
                return description;
            }
            set
            {
                description = value;
                RaisePropertyChanged("Description");
            }
        }        

        private void InitializeCommands()
        {
            // when someone uses a context menu to delete THIS book,
            // broadcast a message to anyone listening so any affected viewmodels
            // can handle it (e.g the global book list)
            RemoveBookCommand = new RelayCommand(
                () => Messenger.Default.Send<RemoveBookMessage>(new RemoveBookMessage(this)));
        }
    }
}
