﻿using System;
using System.Windows;
using Microsoft.Phone.Controls;
using System.Security.Cryptography;
using System.IO;
using System.Text;
using System.IO.IsolatedStorage;

namespace EncryptionSample
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        private AesManaged GetAesWithKey(string password, string salt)
        {
            AesManaged aes = new AesManaged();
            Rfc2898DeriveBytes keySource = new Rfc2898DeriveBytes(password, Encoding.UTF8.GetBytes(salt));
            aes.Key = keySource.GetBytes(aes.KeySize / 8);
            aes.IV = keySource.GetBytes(aes.BlockSize / 8);
            return aes;
        }

        private string EncryptString(string secretText, string password, string salt)
        {
            AesManaged aes = GetAesWithKey(password, salt);            
            MemoryStream ms = new MemoryStream();
            // don't even want the memory storage to be exposed, use crypto stream
            // crypto stream encrypts as data is written to it. 
            CryptoStream cs = new CryptoStream(ms, aes.CreateEncryptor(), CryptoStreamMode.Write);
            byte[] secretBytes = Encoding.UTF8.GetBytes(secretText);
            cs.Write(secretBytes, 0, secretBytes.Length);
            cs.FlushFinalBlock();
            byte[] encryptedBytes = ms.ToArray();
            ms.Close();
            cs.Close();
            return Convert.ToBase64String(encryptedBytes);
        }

        private string DecryptString(string encryptedText, string password, string salt)
        {
            AesManaged aes = GetAesWithKey(password, salt);
            MemoryStream ms = new MemoryStream();
            CryptoStream cs = new CryptoStream(ms, aes.CreateDecryptor(), CryptoStreamMode.Write);
            //byte[] secretBytes = Encoding.UTF8.GetBytes(encryptedText);
            byte[] secretBytes = Convert.FromBase64String(encryptedText);
            cs.Write(secretBytes, 0, secretBytes.Length);
            cs.FlushFinalBlock();
            byte[] decryptedBytes = ms.ToArray();
            return Encoding.UTF8.GetString(decryptedBytes, 0, decryptedBytes.Length);
        }

        private void decryptButton_Click(object sender, RoutedEventArgs e)
        {
            if (!IsolatedStorageSettings.ApplicationSettings.Contains("secret"))
                MessageBox.Show("You haven't stored a secret string in isolated storage yet.");
            else
            {
                string encrypted = IsolatedStorageSettings.ApplicationSettings["secret"].ToString();
                string decrypted = DecryptString(encrypted, passwordBox.Password, saltBox.Password);
                MessageBox.Show("Decrypted secret text from Iso Storage:\n" + decrypted);
            }
        }

        private void encryptButton_Click(object sender, RoutedEventArgs e)
        {
            string encrypted = EncryptString(secretDataTextBox.Text, passwordBox.Password, saltBox.Password);

            IsolatedStorageSettings.ApplicationSettings["secret"] = encrypted;

            string decrypted = DecryptString(encrypted, passwordBox.Password, saltBox.Password);
            MessageBox.Show("Encrypted text:\n" + encrypted + "\n" +
                "Decrypted again:\n" + decrypted + "\n");
        }
    }
}