﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Diagnostics;

namespace WindowsPhoneApplication1
{
    public delegate void OnGameStartedDelegate(DateTime startTime);
    public delegate void OnGamePausedDelegate();
    public delegate void OnGameFinishedDelegate();

    public static class GameEngine
    {
        public static void Start()
        {
            if (OnGameStarted != null)
                OnGameStarted(DateTime.Now);
        }

        public static void Stop()
        {
            if (OnGameFinished != null)
                OnGameFinished();
        }

        public static void Pause()
        {
            if (OnGamePaused != null)
                OnGamePaused();
        }

        public static void Resume()
        {
            if (OnGameResumed != null)
                OnGameResumed(DateTime.Now);
        }

        public static Combatant CreateCombatant()
        {
            Combatant c = new Combatant();
            c.OnGotKilled += new DidGetKilledDelegate(c_OnGotKilled);
            c.OnHitByCombatant += new DidHitTargetDelegate(c_OnHitByCombatant);
            c.OnHitCombatant += new DidHitTargetDelegate(c_OnHitCombatant);

            return c;
        }

        static void c_OnHitCombatant(Combatant attacker, Combatant target, int hitPoints)
        {
            BroadCast(attacker.Name + " hit " + target.Name + " for " + hitPoints.ToString() + " hitpoints.");
        }

        static void c_OnHitByCombatant(Combatant attacker, Combatant target, int hitPoints)
        {
            BroadCast(target.Name + " was hit for " + hitPoints.ToString() + " by " + attacker.Name);
        }

        static void c_OnGotKilled(Combatant killer, Combatant target)
        {
            BroadCast(killer.Name + " killed " + target.Name + "! OH NOEZ!!1ONE");
        }

        static void BroadCast(string msg)
        {
            // ordinarily there would be network code here
            // to broadcast to the entire game

            Debug.WriteLine("[Game]: " + msg);
        }

        public static event OnGameStartedDelegate OnGameStarted;
        public static event OnGameFinishedDelegate OnGameFinished;
        public static event OnGamePausedDelegate OnGamePaused;
        public static event Action<DateTime> OnGameResumed;

    }
}
